/*
    CHAMELEON File Encryption
    Copyright (C) 2000	Ulli Meybohm, www.meybohm.de (ulli@meybohm.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include "chameleon.h"
#define KEYLENGTH 256




void chameleon_generate_key(unsigned char* password, unsigned char* key){
	int keyindex;
	int passwordsize=strlen(password);
	if (passwordsize>KEYLENGTH) passwordsize=KEYLENGTH-1;
	for (keyindex=0; keyindex<KEYLENGTH; keyindex++) {
		key[keyindex]=password[keyindex % passwordsize];
	}
	for (keyindex=0; keyindex<KEYLENGTH; keyindex++) {
		key[keyindex]^=(keyindex*( password[keyindex % passwordsize] * password[(KEYLENGTH-keyindex)%passwordsize])) % 256;
	}
}



unsigned char chameleon_init_modifier(unsigned char * key) {
	int keyindex;
	unsigned char result=0;
	for (keyindex=0; keyindex<KEYLENGTH; keyindex++) result^=key[keyindex];
	return (result);
}




chameleon_t* chameleon_init(char * password, int mode){
	chameleon_t *c;
	c=(chameleon_t*) malloc(sizeof(chameleon_t));
	c->key=(unsigned char*)malloc(sizeof(char)*KEYLENGTH);
	chameleon_generate_key(password,c->key);
	c->keyindex=0;
	c->modifier=chameleon_init_modifier(c->key);
	c->mode=mode;
	return c;
}


char chameleon_char(chameleon_t* c, char source){
	unsigned char inputchar=(unsigned char)source;
	unsigned char outputchar=(unsigned char)
		inputchar^ 
		c->key[c->keyindex]^ 
		c->key[(c->keyindex+123)%256]^ 
		c->key[c->key[c->keyindex]]^ 
		c->modifier;

	if (c->mode==ENCODINGMODE){
		c->modifier= (c->modifier+inputchar) % 256;
		c->key[KEYLENGTH-1-c->keyindex]^=
			c->key[c->key[KEYLENGTH-1-c->keyindex]] ^
			inputchar;
		c->key[c->key[KEYLENGTH-1-c->keyindex]]^=c->modifier;
	} else {
		c->modifier= (c->modifier+outputchar) % 256;
		c->key[KEYLENGTH-1-c->keyindex]^=
			c->key[c->key[KEYLENGTH-1-c->keyindex]] ^ 
			outputchar;
		c->key[c->key[KEYLENGTH-1-c->keyindex]]^=c->modifier;
	}
	c->keyindex++;
	c->keyindex%=KEYLENGTH;
	return (char)outputchar;
}


void chameleon_free(chameleon_t* cstruct) {
	free(cstruct->key);
}



void chameleon_stream(FILE* inputstream , FILE* outputstream, char * password, int mode){
	chameleon_t *c;
	char input;
	c=chameleon_init(password, mode);
	while(EOF!=(input=fgetc(inputstream))){
		fputc( chameleon_char(c,(char)input) , outputstream );
	}
	chameleon_free(c);
}
	

void chameleon_dh_stream
(FILE * inputstream , FILE * outputstream, char * password, int mode,
int headersize, char * randominit) {

	chameleon_t *c;
	int input;
	chameleon_t *rand;
	int headerindex=0;
	int randomlen;
	char randomout;
	int keyindex;
	unsigned int modifier=headersize;
	c=chameleon_init(password, mode);
	
	/*
		Now, let's mix the standard chameleon-key somehow
		with the headersize PIN-number, to test the
		brute-force-attacker's CPU-Power ;-)
	*/
	c->modifier^=(modifier % 256);
	for ( keyindex=0; keyindex<KEYLENGTH; keyindex++) {
		c->key[keyindex]^= (modifier % (keyindex+c->key[KEYLENGTH-1-keyindex])) % 256;
	}
	
	/*Generate/Ignore the dummy-header*/
	if (mode==ENCODINGMODE) {
		/* ON ENCODINGMODE */
		/* Generate some random data-junk for the dummy-header .
		   We can use the chameleon-encryption-process to gernerate random
		   data-junk because the encryption-process behaves chaotically!
		*/
		rand=chameleon_init(randominit, ENCODINGMODE);
		randomlen=strlen(randominit);
		while( headerindex < headersize ) {
			randomout=chameleon_char(rand,randominit[headerindex%randomlen]);
			fputc(chameleon_char(c,randomout),outputstream);
			headerindex++;
		}
		chameleon_free(rand);
	} else { 
		/* ON DECODINGMODE */
		/*Ignore the dummy-header on decryption*/
		while (headersize--) {
			input=fgetc(inputstream);
			chameleon_char(c,(char)input);
			/* The dummy-bytes must also be decrypted, because
			   the chameleon-key in c* will be modified during the
			   decoding process, depending on the decoded plaintext.
			   Althought the result is junk, the key-modification
			   is nessecary and increases the security level.
			*/
		}
	}
	
	/* Now starts the real decryption... */
	while(EOF!=(input=fgetc(inputstream))){
		fputc( chameleon_char(c,(char)input) , outputstream );
	}
	chameleon_free(c);
}
