/*
    CHAMELEON File Encryption
    Copyright (C) 2000	Ulli Meybohm, www.meybohm.de (ulli@meybohm.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "chameleon.h"

#define CVER "0.1"
#define TRUE 1
#define FALSE 0

#ifndef DECODE
#ifndef ENCODE
#define ENCODE
#endif
#endif

#ifndef ENCSUFFIX
#define ENCSUFFIX ".enc"
#endif

#ifndef DECSUFFIX
#define DECSUFFIX ".dec"
#endif



char * password;
char * randominit;
int headersize=10000;
int error=0;



char * stringcopy(char * source){
	int strsize=strlen(source)+1; /* + \0 */
	char *result=(char *) malloc(strsize*sizeof(char));
	strcpy(result,source);
	return result;
}

char * stringappend(char * source, char * app){
	int strsize=strlen(source)+strlen(app)+1; /* + \0 */
	char *result=(char *) malloc(strsize*sizeof(char));
	strcpy(result,source);
	strcat(result,app);
	return result;
}


int stringfind(char *s,char *find, int startpos){
	int size=strlen(s);
	int fsize=strlen(find);
	int eq;
	int i,j;
	for (i=startpos; i<(size-fsize+1); i++) {
		if (toupper(s[i])==toupper(find[0])) {
			eq=TRUE;
			for (j=0; j<fsize; j++) {
		        	eq = eq && (toupper(find[j])==toupper(s[i+j]));
		        	if (!eq) j=fsize;
			}
			if (eq) {
				return i;
			}
		}
	}
	return -1;
}


/* generate some more or less random results */
void string_crasher(char * dest , char *src) {
	int i;
	int ldest=strlen(dest);
	int lsrc=strlen(src);
	for ( i=0; i<ldest; i++) {
		dest[i]=dest[i]^src[i%lsrc]^i^((char)(src[i]*i%256));
	}
}


void parameter_error(){
#ifdef ENCODE
	printf("ccenc %s - CHAMELEON File Encryption\n",CVER);
	printf("Start  : ccenc <files>\n");
#else
	printf("ccdec %s - CHAMELEON File Decryption\n",CVER);
	printf("Start  : ccdec <files>\n");
#endif
	printf("Check www.meybohm.de for updates\n");
	printf("(C) Ulli Meybohm, 2000. License: GNU General Public License\n");
	exit(1);
}


void readpassword(){
	int header;
	int steps=0;
#ifdef ENCODE
	steps=3;
	printf("\n-<CHAMELEON FILE ENCRYPTION>- \n\n");
#else
	steps=2;
	printf("\n-<CHAMELEON FILE DECRYPTION>-\n\n\n");
#endif
	printf("PASSWORD (Step 1/%d)\n",steps);
#ifdef ENCODE
	printf("Enter a password, from which the internal 2024bit-key will be generated.\n");
	printf("For a higher security, use also numbers and special charachters.\n");
	printf("Read the documentation for more information about secure passwords.\n");
	printf("The password is case-sensetive and should be at least 6-8 bytes long.\n");
	printf("Check the password with the vpass utility first if you are unsure.\n");
#endif
	printf("\nEnter Password:");
	password=(char*)malloc(4096*sizeof(char));
	scanf("%s",password);
	password[256]=0;
	printf("\n\nDUMMY-HEADER SIZE (Step 2/%d)\n(Can also be used as additional PIN)\n",steps);
	printf("The dummy-header is used to make known-plaintext/header attacks and\n");
	printf("brute-force attacks much more difficult).\n");
	printf("Enter the size in bytes of the header. [0-n] 0=default=10000\n");
#ifdef ENCODE
	printf("Usually, a number between [1000-10000] should be o.k.\n");
	printf("The longer the header, the more secure is the encryption, but the\n"); 
	printf("bigger is the encrypted file. (encryptedfilesize=headersize+plaintextsize).\n");
	printf("Choose a number between [100000-1000000] if you want to mess with the NSA.\n");
	printf("Please remember, that you need to know this number to decrypt the file!\n");
	printf("If you use the default value \"0\", the encryption is still secure, but not as\n");
	printf("secure as the the encryption with a secret dummy-header-size value.\n");
#endif
	printf("If you are unsure or if you haven't specified a number with your\n"); 
	printf("communication-partner then just type 0. (Read the docs for more information).\n");
	printf("Enter Dummy-Header Size (PIN) [0-n]:"); 
	scanf("%d",&header);
	header?(headersize=header):(headersize=10000);
#ifdef ENCODE
	printf("\n\nRANDOM GERNERATOR INITIALISATION (Step 3/%d)\n",steps);
	printf("To create the dummy-header, we need to create some data-junk with a random\n"); 
	printf("number generater. To initialise the generator, type about 20-30 (or more)\n");
	printf("characters on your keyboard and press RETURN to start the encryption process.\n");
	printf("Type something random:");
	randominit=(char*)malloc(4096*sizeof(char));
	scanf("%s",randominit);
	randominit[256]=0;
#endif
	printf("\n");
}


#ifdef ENCODE
void encodefile(char *filename) {
	char *outputfilename=stringappend(filename,ENCSUFFIX);
	FILE *in;
	FILE *out;
	printf("Encoding %s. Please wait...\n",outputfilename);
	in=fopen(filename,"rb");
	if (in==NULL) {
		printf("Error: Can't open %s!",filename);
		error=1;
        return;
	}
	out=fopen(outputfilename,"w+b");
    if (out==NULL) {
		free(outputfilename);
      	fclose(in);
       	printf("Error: Can't create %s!",outputfilename);
		error=1;
       	return;
	}
	/*ENCRYPT:*/
	chameleon_dh_stream(in,out,password,ENCODINGMODE,headersize,randominit);
	printf("DONE\n");
	fclose(in);
	fclose(out);
	free(outputfilename);
}
#else
void decodefile(char *filename) {
	int i,suffixpos=-1;
	char *outputfilename;
	FILE *in;
	FILE *out;
	printf("Decoding %s. Please wait...\n",filename);
	in=fopen(filename,"rb");
	if (in==NULL) {
		printf("Error: Can't open %s!",filename);
		error=1;
        return;
	}
	/* Get outputfile name  ( replace ".enc" with "" ) */
	if ( -1 != (suffixpos=stringfind(filename,ENCSUFFIX,0)) ) {
		while ( -1 != (i=stringfind(filename,ENCSUFFIX,suffixpos+1)) ){
			suffixpos=i;		
		}
		outputfilename=stringcopy(filename);
		outputfilename[suffixpos]=0;
	} else {
		outputfilename=stringappend(filename,DECSUFFIX);
	}
	out=fopen(outputfilename,"w+b");
    if (out==NULL) {
      	fclose(in);
       	printf("Error: Can't create %s!",outputfilename);
		free(outputfilename);
		error=1;
       	return;
	}
	/*DECRYPT:*/
	chameleon_dh_stream(in,out,password,DECODINGMODE,headersize,NULL);
	printf("DONE\n");
	fclose(in);
	fclose(out);
	free(outputfilename);
}
#endif



void run(int filecount, char* files[]){
	int i;
   	readpassword();
	for (i=0; i<filecount; i++) {
		if (strlen(files[i])>0) {
#ifdef ENCODE
			/* 
				to prevent, that every file of files[i]
				gets the same random header, we must massively modify 
				the randomizer initialisation string with
				the filname-string, using the string_crasher function
			*/
			string_crasher(randominit, files[i]);
			encodefile(files[i]);
#else
			decodefile(files[i]);
#endif		
		}
	}
}

int main(int argc,char* argv[]){
	argc-=1;
	argv+=1;
	if (argc>0) {
    	run(argc,argv);
 	} else 	{
		parameter_error();
		error=1;
	}
	return (error);
}
